@@ -1,11 +1,11 @@
 import os
 import time
+import json
 import types
-import threading
 import numpy as np
 
 from enum import Enum
-from typing import List, AnyStr
+from typing import List
 
 from cnvrg.modules.project import Project
 from cnvrg.modules.errors import UserError
@@ -17,7 +17,7 @@ from cnvrg.helpers.libs_helper import check_if_lib_exists
 from cnvrg.modules.mixins.charts_mixin import ChartsMixin
 from cnvrg.helpers.error_catcher import suppress_exception
 from cnvrg.helpers.param_helper import wrap_string_to_list
-from cnvrg.modules.cnvrg_job import CnvrgJob, LOGS_TYPE_OUTPUT, LOGS_TYPE_ERROR
+from cnvrg.modules.cnvrg_job import CnvrgJob, LOGS_TYPE_ERROR
 
 import cnvrg.helpers.env_helper as env_helper
 import cnvrg.helpers.apis_helper as apis_helper
@@ -72,12 +72,15 @@ class Experiment(CnvrgJob, ChartsMixin):
         env_helper.set_current_job_id(self.job_slug)
         env_helper.set_current_job_type("Experiment")
 
-
     @staticmethod
-    def __run_callable(function_command, title=None, project=None, cwd=None, library=None, **kwargs):
+    def __run_callable(function_command, title=None, project=None, library=None, **kwargs):
         experiment = Experiment._create_clean_experiment(title=title, project=project, library=library, **kwargs)
-        exit_status = control_stdout_helper.\
-            run_callable(function_command, arguments=[experiment], callback=lambda x: experiment.log(x), err_callback=lambda x: experiment.log(x, log_type=LOGS_TYPE_ERROR))
+        exit_status = control_stdout_helper.run_callable(
+            function_command,
+            arguments=[experiment],
+            callback=lambda x: experiment.log(x),
+            err_callback=lambda x: experiment.log(x, log_type=LOGS_TYPE_ERROR)
+        )
         experiment.finish(exit_status=exit_status)
         return experiment
 
@@ -85,11 +88,11 @@ class Experiment(CnvrgJob, ChartsMixin):
     def __run_local(cmd, **kwargs):
         def function_command(experiment=None):
             env = None
-            if experiment: env = experiment.as_env()
-            return spawn_helper.run_sync(cmd, print_output=True, cwd=kwargs["cwd"], env=env)
+            if experiment:
+                env = experiment.as_env()
+            return spawn_helper.run_sync(cmd, print_output=True, cwd=kwargs["cwd"], env=env)["code"]
         return Experiment.__run_callable(function_command, cmd=cmd, **kwargs)
 
-
     def as_env(self):
         return {
             env_helper.ENV_KEYS["current_job_id"]: self.job_slug,
@@ -98,14 +101,16 @@ class Experiment(CnvrgJob, ChartsMixin):
             env_helper.ENV_KEYS["current_organization"]: self.project.owner,
         }
 
-    def sync(self, output_dir='output'):
+    def sync(self, output_dir='output', git_diff=False):
         payload = {
             "job_type": "Experiment",
             "job": self.job_slug,
-            "in_exp": "true",
+            "in_exp": "true"
         }
         if self.project.is_git():
             payload["output_dir"] = output_dir
+            if git_diff:
+                payload["git_diff"] = git_diff
 
         self.project.sync(**payload)
 
@@ -178,28 +183,46 @@ class Experiment(CnvrgJob, ChartsMixin):
         return mapped_files
 
     @staticmethod
-    def     run(command=None, arguments: dict=None, grid: dict=None, title: str=None, project: Project=None, compute: str=None, output_dir: str=None, computes: List=None,
-            datasets: List=None, dataset: str=None, local: bool=False, library: str=None, working_directory: str=None, sync_before: bool=True, sync_after: bool=True,
-            prerun: bool=True, requirements: bool=True, image: str=None, schedule: str=None, recurring: str=None, git_commit: str=None, git_branch: str=None,
-            local_folders: List=None, notify_on_error: bool=False, notify_on_success: bool=False, emails_to_notify: List=None, **kwargs):
+    def run(command=None, arguments: dict=None, grid: dict=None, title: str=None, project: Project=None,
+            compute: str=None, output_dir: str=None, computes: List=None,datasets: List=None, dataset: str=None,
+            local: bool=False, library: str=None, working_directory: str=None, sync_before: bool=True,
+            sync_after: bool=True, prerun: bool=True, requirements: bool=True, image: str=None, schedule: str=None,
+            recurring: str=None, git_commit: str=None, git_branch: str=None, local_folders: List=None,
+            notify_on_error: bool=False, notify_on_success: bool=False, emails_to_notify: List=None,
+            git_diff=False, commit: str=None, **kwargs):
         function_command = None
         project = project or Project()
-        if sync_before == None: sync_before = not project.git
+        if sync_before is None:
+            sync_before = not project.git or git_diff
         if project.git:
             git_commit = git_commit or project.get_git_commit()
             output_dir = output_dir or "output"
             git_branch = git_branch or project.get_git_branch()
 
         ## sync before
-        if sync_before: project.sync(**CnvrgJob.current_job_sync_args(output_dir=output_dir))
+        if sync_before:
+            sync_output = project.sync(
+                get_output_by_regex='\{.*\"commit_sha1\"\:\".*\"\}',
+                **CnvrgJob.current_job_sync_args(output_dir=output_dir, git_diff=git_diff, return_id=True)
+            )["output"]
+            if len(sync_output) > 0 and project.git:
+                sync_output = json.loads(sync_output[0])
+                if sync_output["commit_sha1"]:
+                    commit = sync_output["commit_sha1"]
+
         ## support
         computes = computes or wrap_string_to_list(computes) or wrap_string_to_list(compute) or []
         ### by default (if computes empty or no compute) set the local to True
-        if local: pass
-        elif "local" in computes: local = True
-        elif arguments: local = False
-        elif grid: local = False
-        elif len(computes) == 0: local = True
+        if local:
+            pass
+        elif "local" in computes:
+            local = True
+        elif arguments:
+            local = False
+        elif grid:
+            local = False
+        elif len(computes) == 0:
+            local = True
 
         ### if command is function, run it as a function and not as a os command
         if isinstance(command, types.FunctionType): function_command = command
@@ -211,19 +234,58 @@ class Experiment(CnvrgJob, ChartsMixin):
             if function_command:
                 ## set the title to "func: {name of the function}"
                 cmd = "func: {func_name}".format(func_name=function_command.__name__)
-                experiment = Experiment.__run_callable(function_command, title=title, project=project, library=library, cmd=cmd, git_commit=git_commit, output_dir=output_dir, git_branch=git_branch)
-                if not experiment: raise UserError("Cant run experiment, please check your input params")
+                experiment = Experiment.__run_callable(
+                    function_command,
+                    title=title,
+                    project=project,
+                    library=library,
+                    cmd=cmd,
+                    git_commit=git_commit,
+                    output_dir=output_dir,
+                    git_branch=git_branch,
+                    commit=commit
+                )
+                if not experiment:
+                    raise UserError("Cant run experiment, please check your input params")
             else:
-                experiment = Experiment.__run_local(cmd=command, title=title, project=project, cwd=working_directory, git_commit=git_commit, git_branch=git_branch,
-                                                    library=library)
+                experiment = Experiment.__run_local(
+                    cmd=command,
+                    title=title,
+                    project=project,
+                    cwd=working_directory,
+                    git_commit=git_commit,
+                    git_branch=git_branch,
+                    library=library,
+                    commit=commit
+                )
                 if not experiment: raise UserError("Cant run experiment, please check your input params")
         else:
             ## perform an api call to create it.
             Experiment.__verify_remote_params(function_command)
-            e = project.run_task(command, arguments=arguments, title=title, templates=computes, datasets=datasets,
-                                 library=library,  prerun=prerun, requirements=requirements,
-                                 grid=grid,git_branch=git_branch, git_commit=git_commit, output_dir=output_dir, image=image, schedule=schedule, recurring=recurring,
-                                 local_folders=local_folders, notify_on_error=notify_on_error, notify_on_success=notify_on_success, emails_to_notify=emails_to_notify, **kwargs)
+            e = project.run_task(
+                command,
+                arguments=arguments,
+                title=title,
+                templates=computes,
+                datasets=datasets,
+                library=library,
+                prerun=prerun,
+                requirements=requirements,
+                grid=grid,
+                git_branch=git_branch,
+                git_commit=git_commit,
+                output_dir=output_dir,
+                image=image,
+                schedule=schedule,
+                recurring=recurring,
+                local_folders=local_folders,
+                notify_on_error=notify_on_error,
+                notify_on_success=notify_on_success,
+                emails_to_notify=emails_to_notify,
+                working_directory=working_directory,
+                commit=commit,
+                **kwargs
+            )
             if len(e.get("experiments")) == 1:
                 exp = Experiment(e.get("experiments")[0], owner=project.owner)
                 exp.notify_running()
