@@ -36,7 +36,7 @@ class Project(CnvrgFiles):
 
     def __fetch_config(self):
         resp = apis_get(self.get_base_url())
-        return {"slug": self.slug, "owner": self.owner, **resp.get("result")}
+        return {"slug": self.slug, "owner": self.owner, **json.loads(resp.get("result"))}
 
     def is_git(self):
         resp = apis_get(url_join(self.get_base_url(), 'get_project'))
@@ -210,7 +210,6 @@ class Project(CnvrgFiles):
             raise CnvrgError(error_msg)
         return hyper_search
 
-
-    def sync(self, **options):
+    def sync(self, get_output_by_regex=".*", **options):
         command = "cnvrg sync {args}".format(args=args_helper.args_to_string(options))
-        spawn_helper.run_sync(command, print_output=True)
\ No newline at end of file
+        return spawn_helper.run_sync(command, print_output=True, get_output_by_regex=get_output_by_regex)
